#!/usr/bin/env bash


function build_conjecture()
{
    g++ -std=c++0x -Ofast -flto -Wall -pedantic \
        common/*.cpp conjecture/*.cpp -o conject &&
    strip -s conject
}


function build_precounts()
{
    g++ -std=c++0x -Ofast -flto -Wall -pedantic \
        common/*.cpp precounts/*.cpp -o counter &&
    strip -s counter
}


function gen-precounts()
{
    ./counter sjis texts/*sjis* > conjecture/PreCountsSJIS.txt
    ./counter utf8 texts/*utf8* > conjecture/PreCountsUTF8.txt
}


if [[ "$1" == "" || "$1" == "conjecture" ]]
then
    build_conjecture

elif [[ "$1" == "precounts" ]]
then
    build_precounts

elif [[ "$1" == "gen-precounts" ]]
then
    gen-precounts

elif [[ "$1" == "distclean" ]]
then
    rm -f conject counter

else
    echo "$0" >&2
    echo "     builds the main program 'conject'." >&2
    echo "$0 precounts" >&2
    echo "     builds a tool 'counter'" >&2
    echo "     to update conjecture/PreCounts*.txt."  >&2
    echo "$0 gen-precounts" >&2
    echo "     invokes 'precounts' to feed it all texts in texts/" >&2
    echo "     and updates conjecture/PreCounts*.txt." >&2
    echo "   NOTE:" >&2
    echo "     The updated tables don't have effect" >&2
    echo "     unless you rebuild 'conject'." >&2
fi
