#include "CharDigester.h"

#include "CounterSJIS.h"
#include "CounterUTF8.h"

#include <cstring>
#include <stdexcept>


namespace my
{
namespace
{
    template <class Counter>
    class CharDigesterImpl : public CharDigester
    {
    public:
        virtual
        void operator()(unsigned char const* buf, std::size_t n)
        {
            for(std::size_t i = 0; i < n; ++i){
                counter_(buf[i]);
            }
        }

        virtual
        std::uint_fast32_t const (&GetCounter() const)[DIM_CHARS]
        {
            return counter_.GetCounter();
        }

        virtual
        std::uint_fast32_t GetNumErrors() const
        {
            return counter_.GetNumErrors();
        }

        virtual
        void ResetState()
        {
            counter_.ResetState();
        }

    private:
        Counter counter_;
    };
} // anonymous

    CharDigester::Ptr
    CharDigester::Create(char const* encoding)
    {
        if(0 == std::strcmp(encoding, "sjis")){
            return Ptr(new CharDigesterImpl<CounterSJIS>);
        }
        if(0 == std::strcmp(encoding, "utf8")){
            return Ptr(new CharDigesterImpl<CounterUTF8>);
        }

        throw std::runtime_error
        (   std::string("No such encoding: ")
            + encoding
        );
    }
}
