#include "ConjectureEncoding.h"
#include "PreCounts.h"
#include "../common/CharDigester.h"
#include "../common/NormalizeCounts.h"

#include <algorithm>
#include <numeric>
#include <vector>

namespace my
{
    std::size_t
    ConjectureEncoding
    (   unsigned char const* data
    ,   std::size_t          cbData
    ,   char const* const *  encodings
    ,   std::size_t          nEncodings
    ){
        std::vector<CharDigester::Ptr> digesters(nEncodings);

        std::size_t nOKs = 0;
        std::size_t iOK  = 0;

        for(std::size_t i = 0; i < nEncodings; ++i){
            digesters[i] = CharDigester::Create(encodings[i]);
            auto& theDigester = *digesters[i];

            theDigester(data, cbData);
            if(theDigester.GetNumErrors() == 0){
                ++nOKs;
                iOK = i;
            }
        }

        if(nOKs == 1) return iOK;

        std::vector<double> products(nEncodings);
        for(std::size_t i = 0; i < nEncodings; ++i){
            double obs[DIM_CHARS];
            NormalizeCounts(digesters[i]->GetCounter(), obs);

            // product = \vec{observed} \cdot \vec{expected}
            products[i] = std::inner_product
            (   obs, obs+lengthof(obs)
            ,   GetPreCounts(encodings[i])
            ,   (double)0
            );
        }

        auto maxIt = std::max_element(products.begin(), products.end());
        if(*maxIt > 0){
            return maxIt - products.begin();
        }
        else{
            return products.size();
        }
    }

} // namespace my
