#include "ConjectureEncoding.h"
#include "../common/Miscellanea.h"
#include <cstdio>
#include <cstring>
#include <stdexcept>
#include <string>
#include <vector>


namespace my
{
    std::vector<unsigned char>
    LoadFile(char const* szFile)
    {
        std::vector<unsigned char> data(1024*1024);

        std::FILE* fin = std::fopen(szFile, "rb");
        if(! fin){
            throw std::runtime_error("Input file is not accessible");
        }

        std::size_t size = std::fread(data.data(), 1, data.size(), fin);
        std::fclose(fin);

        data.resize(size);
        return data;
    }


    void PrintHelp(char const* szSelfName)
    {
        std::fprintf
        (   stderr
        ,   "%s (-v) [file]\n\n"
            "This program conjectures the text encoding of [file] for SciTE.\n"
            "Without -v option, no error message is shown.\n"
        ,   szSelfName
        );
    }

} // namespace my


int main(int argc, char* argv[])
{
    bool bVerbose = false;
try{
    char const* szFile = nullptr;

    if(argc == 2){
        szFile = argv[1];
    }
    else if(argc == 3){
        if(0 == std::strcmp(argv[1], "-v")){
            bVerbose = true;
            szFile = argv[2];
        }
        else if(0 == std::strcmp(argv[2], "-v")){
            bVerbose = true;
            szFile = argv[1];
        }
    }

    if(! szFile){
        my::PrintHelp(argv[0]);
        return 1;
    }

    char const* const encodings[] = { "utf8", "sjis"};
    int         const codePages[] = { 65001  , 932  };

    auto rawData = my::LoadFile(szFile);
    if(rawData.empty()){
        throw std::runtime_error
        (   std::string("No bytes read from ")
            + szFile
        );
    }

    auto index = my::ConjectureEncoding
    (   rawData.data(), rawData.size()
    ,   encodings, lengthof(encodings)
    );

    if(index < lengthof(encodings)){
        std::printf("code.page=%d\n", codePages[index]);
    }
}
catch(std::exception& e){
    if(bVerbose) std::fprintf(stderr, "error: %s\n", e.what());
    return 1;
}
    return 0;
}
