#include "../common/CharDigester.h"
#include "../common/NormalizeCounts.h"

#include <algorithm>
#include <cstdint>
#include <numeric>
#include <string>


int main(int argc, char* argv[])
try {
    if(argc <= 2){
        std::fprintf(stderr, "%s [ENCODING] FILE...\n\n", argv[0]);
        std::fprintf(stderr,
            "Read FILE... and counts multi-byte chars.\n"
            "Normalized counts (such that \\sum c_i^2 = 1) are output in a C array.\n"
        );

        return 1;
    }

    char const* const encoding = argv[1];

    auto pDigester = my::CharDigester::Create(encoding);
    std::uint_fast32_t nErrors = 0;

    for(int i = 2; i < argc; ++i){
        pDigester->ResetState();

        std::fprintf(stderr, "reading %s...\n", argv[i]);
        (*pDigester)(argv[i]);

        if(nErrors != pDigester->GetNumErrors()){
            auto e = pDigester->GetNumErrors() - nErrors;
            nErrors = pDigester->GetNumErrors();

            std::fprintf(stderr, "warning: there are %u invalid bytes\n", (unsigned)e);
        }
    }

    double normCounts[my::DIM_CHARS];
    my::NormalizeCounts(pDigester->GetCounter(), normCounts);

    std::printf(
        "static double const preCounts_%s[] = {\n", encoding
    );
    for(std::size_t i = 0; i < lengthof(normCounts); ++i){
        std::printf(
            "/* %02x */ %.16g,\n", (int)i, normCounts[i]
        );
    }
    std::printf("};\n");

    return 0;
}
catch(std::exception& e)
{
    std::fprintf(stderr, "error: %s\n", e.what());
    return 1;
}
